package com.wf.lucene;

import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.StringField;
import org.apache.lucene.document.TextField;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopScoreDocCollector;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.Version;

public class LuceneOnFileSystemExample {
	static String DATA_FOLDER = "/home/piyas/Documents/Winkframe/sample_text_files/drugs/";
	static String INDEX_FOLDER = "/home/piyas/Documents/Winkframe/sample_text_files/drugindex/";
	private static StandardAnalyzer analyzer = new StandardAnalyzer(Version.LUCENE_46);
    private static IndexWriter writer;
	private static ArrayList<File> queue = new ArrayList<File>();
	
	public static void indexFilesAndShowResults(String dataFilePath,String indexFilePath,String searchTerm) throws Exception {
		
		// Indexing part
		indexOnThisPath(indexFilePath);
		indexFileOrDirectory(dataFilePath);
		closeIndex();
		
		// Search Part
		searchInIndexAndShowResult(indexFilePath, searchTerm);
		
		
	}
	
	public static void searchInIndexAndShowResult(String indexFilePath,String searchString) throws Exception{
		IndexReader reader = DirectoryReader.open(FSDirectory.open(new File(indexFilePath)));
	    IndexSearcher searcher = new IndexSearcher(reader);
	    TopScoreDocCollector collector = TopScoreDocCollector.create(5, true);

	    
	    Query q = new QueryParser(Version.LUCENE_46, "contents", analyzer).parse(searchString);
	    searcher.search(q, collector);
	    ScoreDoc[] hits = collector.topDocs().scoreDocs;

        // display results
        System.out.println("Found " + hits.length + " hits.");
        for(int i=0;i<hits.length;++i) {
          int docId = hits[i].doc;
          Document d = searcher.doc(docId);
          System.out.println((i + 1) + ". " + d.get("path") + " score=" + hits[i].score);
        }
	}
	
	
	 public static void closeIndex() throws Exception {
		    writer.close();
		  }
	
	public static void indexOnThisPath(String indexDir) throws Exception {
	    // the boolean true parameter means to create a new index everytime, 
	    // potentially overwriting any existing files there.
	    FSDirectory dir = FSDirectory.open(new File(indexDir));
	    IndexWriterConfig config = new IndexWriterConfig(Version.LUCENE_46, analyzer);
	    writer = new IndexWriter(dir, config);
	  
	}
	
	
	/**
	   * Indexes a file or directory
	   * @param fileName the name of a text file or a folder we wish to add to the index
	   * @throws java.io.IOException when exception
	   */
	public static void indexFileOrDirectory(String filePath) throws Exception {
	    //===================================================
	    //gets the list of files in a folder (if user has submitted
	    //the name of a folder) or gets a single file name (is user
	    //has submitted only the file name) 
	    //===================================================
	    addFiles(new File(filePath));
	    
	    int originalNumDocs = writer.numDocs();
	    for (File f : queue) {
	      FileReader fr = null;
	      try {
	        Document doc = new Document();

	        //===================================================
	        // add contents of file
	        //===================================================
	        fr = new FileReader(f);
	        doc.add(new TextField("contents", fr));
	        doc.add(new StringField("path", f.getPath(), Field.Store.YES));
	        doc.add(new StringField("filename", f.getName(), Field.Store.YES));

	        writer.addDocument(doc);
	        System.out.println("Added: " + f);
	      } catch (Exception e) {
	        System.out.println("Could not add: " + f);
	      } finally {
	        fr.close();
	      }
	    }
	    
	    int newNumDocs = writer.numDocs();
	    System.out.println("");
	    System.out.println("************************");
	    System.out.println((newNumDocs - originalNumDocs) + " documents added.");
	    System.out.println("************************");

	    queue.clear();
	  }

	  private static void addFiles(File file) {

	    if (!file.exists()) {
	      System.out.println(file + " does not exist.");
	    }
	    if (file.isDirectory()) {
	      for (File f : file.listFiles()) {
	        addFiles(f);
	      }
	    } else {
	      String filename = file.getName().toLowerCase();
	      //===================================================
	      // Only index text files
	      //===================================================
	      if (filename.endsWith(".htm") || filename.endsWith(".html") || 
	              filename.endsWith(".xml") || filename.endsWith(".txt")) {
	        queue.add(file);
	      } else {
	        System.out.println("Skipped " + filename);
	      }
	    }
	  }
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args)  {
		// TODO Auto-generated method stub
		try{
			indexFilesAndShowResults(DATA_FOLDER,INDEX_FOLDER,"HIV");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	

}
